# -*- coding: utf-8 -*-

""" Module summary description.

More detailed description.
"""

__version__ = '1.4.0'
__author__ = 'Benjamin Pillot'
__email__ = 'benjaminpillot@riseup.net'


try:
    from osgeo import gdal
except ImportError:
    import gdal

GDAL_DEFAULT_DRIVER = gdal.GetDriverByName("Gtiff")



# -*- coding: utf-8 -*-

""" Module summary description.

More detailed description.
"""
from functools import partial

from pyrasta.io_ import ESRI_DRIVER
from pyrasta.io_.files import RasterTempFile, ShapeTempFile, GeojsonTempFile
from pyrasta.tools import _return_raster, _gdal_temp_dataset

try:
    from osgeo import gdal
except ImportError:
    import gdal


@_return_raster
def _clip_raster_by_extent(raster, out_file, bounds):
    """ Clip raster by extent

    Parameters
    ----------
    raster: pyrasta.raster.RasterBase
    out_file: pyrasta.io_.files.RasterTempFile
    bounds: tuple
        boundaries as (minx, miny, maxx, maxy)

    Returns
    -------
    RasterBase

    """

    minx = max(bounds[0], raster.bounds[0])
    miny = max(bounds[1], raster.bounds[1])
    maxx = min(bounds[2], raster.bounds[2])
    maxy = min(bounds[3], raster.bounds[3])

    if minx >= maxx or miny >= maxy:
        raise ValueError("requested extent out of raster boundaries")

    # gdal.Translate projWin as [upper left x, upper left y, lower right x, lower right y]
    gdal.Translate(out_file,
                   raster._gdal_dataset,
                   projWin=[minx, maxy, maxx, miny])

    # gdal.Warp(out_file,
    #           raster._gdal_dataset,
    #           outputBounds=bounds,
    #           srcNodata=raster.no_data,
    #           dstNodata=no_data,
    #           outputType=raster.data_type)


def _clip_raster_by_mask(raster, geodataframe, out_no_data, all_touched,
                         window_size, out_data_type, driver, description):
    """ Clip raster by mask from geographic layer

    Parameters
    ----------
    raster: pyrasta.raster.RasterBase
        raster to clip
    geodataframe: geopandas.GeoDataFrame or gistools.layer.GeoLayer
    out_no_data: float or int
        No data value in output raster
    all_touched: bool
        if True, clip all pixels that are touched, otherwise clip
        if pixel's centroids are within boundaries
    window_size: int or list[int, int]
        Size of window for raster calculation
    out_data_type: int
        Output data type for masked raster
    driver
    description: str

    Returns
    -------
    RasterBase

    """
    clip_raster = raster.clip(bounds=geodataframe.total_bounds)

    if driver == "ESRI Shapefile":
        temp_file = ShapeTempFile
    else:
        temp_file = GeojsonTempFile

    with temp_file() as tempfile, \
            RasterTempFile(clip_raster._gdal_driver.GetMetadata()['DMD_EXTENSION']) as r_file:

        geodataframe.to_file(tempfile.path, driver=driver)

        out_ds = _gdal_temp_dataset(r_file.path,
                                    clip_raster._gdal_driver,
                                    clip_raster._gdal_dataset.GetProjection(),
                                    clip_raster.x_size,
                                    clip_raster.y_size,
                                    clip_raster.nb_band,
                                    clip_raster.geo_transform,
                                    clip_raster.data_type,
                                    clip_raster.no_data)

        gdal.Rasterize(out_ds,
                       tempfile.path,
                       bands=list(range(1, clip_raster.nb_band + 1)),
                       burnValues=[1] * clip_raster.nb_band,
                       allTouched=all_touched)

    out_ds = None

    return clip_raster.__class__.raster_calculation([clip_raster,
                                                     clip_raster.__class__(r_file.path)],
                                                    partial(mask_clip, no_data=out_no_data),
                                                    input_type=out_data_type,
                                                    output_type=out_data_type,
                                                    no_data=out_no_data,
                                                    description=description,
                                                    window_size=window_size,
                                                    nb_processes=1,
                                                    chunksize=1)


def mask_clip(arrays, no_data):

    result = arrays[0] * arrays[1]
    result[arrays[1] != 1] = no_data

    return result



# -*- coding: utf-8 -*-

""" Module summary description.

More detailed description.
"""
import numpy as np
# import osmnx as ox
from gistools.layer import PolygonLayer
from matplotlib import pyplot

from pyrasta.raster import DigitalElevationModel, Raster
from pyrasta.tools.clip import _clip_raster_by_mask
from pyrasta.tools.rasterize import _rasterize
from pyrasta.tools.srtm import from_cgiar_online_database

# bounds = (5, 40, 10, 45)
# test = from_cgiar_online_database(bounds)
# test.to_file("/home/benjamin/srtm_corse.tif")

# test = DigitalElevationModel("/home/benjamin/dem_test.tif").clip((13, 42, 15, 45)).to_file(
#     "/home/benjamin/dem_clip_test.tif")
# pop = Raster("/home/benjamin/Documents/PRO/PRODUITS/POPULATION_DENSITY/001_DONNEES/COTE_D_IVOIRE"
#              "/population_civ_2019-07-01_geotiff/population_civ_2019-07-01.tif")
from pyrasta.tools.stats import _zonal_stats

dem = DigitalElevationModel("/home/benjamin/Documents/PRO/PRODUITS/TESTS/dem_ci.tif")
test = dem.to_crs(32630).slope("degree")
test.to_file("/home/benjamin/dem_slope_ci.tif")

country = PolygonLayer.from_gpd(ox.geocode_to_gdf(
    dict(country="Cote d'Ivoire",
         admin_level=2,
         type="boundary"))).to_crs(32630).clean_geometry()

# dem = from_cgiar_online_database(country.total_bounds)
# dem.to_file("/home/benjamin/dem_ci.tif")

honeycomb = country.split(country.area[0] / 100, method="hexana", show_progressbar=True)

honeycomb = honeycomb.to_crs(dem.crs)
honeycomb.to_file("/home/benjamin/Documents/PRO/PRODUITS/TESTS/honeycomb.shp")
honeycomb["ID"] = honeycomb.index

# test = dem.clip(mask=honeycomb[[15]], all_touched=True)
# test.to_file("/home/benjamin/pop.tif")

# test = dem.zonal_stats(honeycomb, stats=["mean", "median"])

# test = dem.rasterize(honeycomb, dem.projection, dem.x_size, dem.y_size, dem.geo_transform,
#                      attribute="ID", nb_band=1)
# test = _rasterize(DigitalElevationModel, honeycomb, None, "ID", dem._gdal_driver,
#                   dem._gdal_dataset.GetProjection(),
#                   dem.x_size, dem.y_size, 1, dem.geo_transform, dem.data_type, -999, True)
# test = DigitalElevationModel(test.path)
# test.to_file("/home/benjamin/test_ci.tif")

stats = _zonal_stats(dem, honeycomb, 1, ['mean', 'median', 'max', 'min'], dict(mymax=np.max), -999,
                     True, True, 6)

print(stats.keys())
print("mymax: %s" % stats["mymax"])
print(len(honeycomb))
